(function() {
  var ck, config, connectESI, express, fs, help, pantry, test;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  express = require('express');
  pantry = require('pantry');
  fs = require('fs');
  connectESI = require('connect-esi');
  ck = require('coffeekup');
  config = {
    appName: "Timbits",
    engine: "coffee",
    port: 5678,
    home: process.cwd()
  };
  this.serve = function(options) {
    var key, path, value;
    for (key in options) {
      value = options[key];
      config[key] = value;
    }
    this.server = express.createServer();
    this.server.register('.coffee', require('coffeekup'));
    this.server.set('views', "" + config.home + "/views");
    this.server.set('view engine', config.engine);
    this.server.set('view options', {
      layout: false
    });
    this.server.configure(__bind(function() {
      this.server.use(connectESI.setupESI());
      this.server.use(express.static("" + config.home + "/public"));
      this.server.use(express.bodyParser());
      return this.server.use(express.cookieParser());
    }, this));
    this.server.configure('development', __bind(function() {
      return this.server.use(express.errorHandler({
        dumpExceptions: true,
        showStack: true
      }));
    }, this));
    this.server.configure('production', __bind(function() {
      return this.server.use(express.errorHandler());
    }, this));
    this.server.get('/', function(req, res) {
      return res.redirect('/timbits/help');
    });
    this.server.get('/timbits/help', __bind(function(req, res) {
      return res.send(ck.render(help, {
        context: this.box
      }));
    }, this));
    this.server.get('/timbits/test', __bind(function(req, res) {
      return res.send(ck.render(test, {
        context: null
      }));
    }, this));
    path = "" + config.home + "/timbits";
    fs.readdir(path, __bind(function(err, files) {
      var file, name, _i, _len, _results;
      if (err) {
        throw err;
      }
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        name = file.replace('.coffee', '');
        _results.push(this.add(name, require("" + path + "/" + name)));
      }
      return _results;
    }, this));
    this.server.listen(process.env.PORT || config.port);
    console.log("Timbits server listening on port " + (this.server.address().port) + " in " + this.server.settings.env + " mode");
    return this.server;
  };
  this.box = {};
  this.add = function(name, timbit) {
    console.log("\t Placing " + name + " in the box");
    timbit.name = name;
    this.box[name] = timbit;
    this.server.get("/" + name + "/help", function(req, res) {
      return res.send(ck.render(timbit.help, {
        context: timbit
      }));
    });
    this.server.get("/" + name + "/test", function(req, res) {
      return res.send(ck.render(test, {
        context: null
      }));
    });
    return this.server.get("/" + name + "/:view?", function(req, res) {
      var attr, context, k, key, v, value, _base, _ref, _ref2, _ref3, _ref4;
      context = {};
      _ref = req.query;
      for (k in _ref) {
        v = _ref[k];
        context[k] = v;
      }
      context.name = name;
      context.view = (_ref2 = (_base = req.params).view) != null ? _ref2 : _base.view = 'default';
      _ref3 = timbit.params;
      for (key in _ref3) {
        attr = _ref3[key];
        if ((_ref4 = context[key]) == null) {
          context[key] = attr["default"];
        }
        value = context[key];
        if (attr.required && !value) {
          throw "" + key + " is a required parameter";
        }
        if (value && attr.strict && attr.values.indexOf(value) === -1) {
          throw "" + value + " is not a valid value for " + key + ".  Must be one of [" + (attr.values.join()) + "]";
        }
        if (value instanceof Array && !attr.multiple) {
          throw "" + key + " must be a single value";
        }
      }
      return timbit.eat(req, res, context);
    });
  };
  help = function() {
    h1('Timbits - Help');
    return ul(function() {
      var k, v, _results;
      _results = [];
      for (k in this) {
        v = this[k];
        _results.push(li(function() {
          return a({
            href: "/" + k + "/help"
          }, function() {
            return k;
          });
        }));
      }
      return _results;
    });
  };
  test = function() {
    return h1('Testing? LOL!!!!!!');
  };
  this.Timbit = (function() {
    function Timbit() {}
    Timbit.prototype.render = function(req, res, context) {
      return res.render("" + context.name + "/" + context.view, {
        context: context
      });
    };
    Timbit.prototype.fetch = function(req, res, context, key, options, callback) {
      if (callback == null) {
        callback = this.render;
      }
      return pantry.fetch(options, __bind(function(error, results) {
        context[key] = results;
        return callback(req, res, context);
      }, this));
    };
    Timbit.prototype.eat = function(req, res, context) {
      return this.render(req, res, context);
    };
    Timbit.prototype.help = function() {
      h1(this.name);
      p(this.about || 'Developer was too lazy to describe this widget');
      h2('Examples:');
      if (this.examples) {
        ul(function() {
          var example, _i, _len, _ref, _results;
          _ref = this.examples;
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            example = _ref[_i];
            _results.push(li(function() {
              return a({
                href: example.href
              }, function() {
                return example.caption;
              });
            }));
          }
          return _results;
        });
      } else {
        p('Developer was too lazy to define any examples.');
      }
      h2('Parameters');
      if (this.params) {
        table({
          border: 1
        }, function() {
          return tbody(function() {
            var attr, key, _ref, _results;
            tr(function() {
              th('Name');
              th('Description');
              th('Type');
              th('Required');
              th('Multiple');
              th('Default');
              return th('Values');
            });
            _ref = this.params;
            _results = [];
            for (key in _ref) {
              attr = _ref[key];
              td(key);
              td(attr.description);
              td(attr.type || 'String');
              td((attr.required || false).toString());
              td((attr.multiple || false).toString());
              td(attr["default"]);
              _results.push(td("" + (attr.strict ? 'One of:' : 'Examples:') + " " + (attr.values.join())));
            }
            return _results;
          });
        });
      } else {
        p('None defined');
      }
      return p(function() {
        return a({
          href: '/timbits/help'
        }, function() {
          return '<< Help Index';
        });
      });
    };
    Timbit.prototype.test = function() {
      return console.log('LOL!');
    };
    return Timbit;
  })();
}).call(this);
